import sys
from os.path import join, dirname


VERSION_FILE = join(dirname(__file__), "VERSION")
with open(VERSION_FILE, 'r') as ver:
    VERSION = ver.readline().strip()

# Needed by setup.py and thus cannot live in Constants as it contains the
# appdirs import that will break setup if appdirs isn't available yet.
BUS_NAME = "org.coala_analyzer.v1"


def assert_supported_version():  # pragma: no cover
    if not sys.version_info > (3, 2):
        print("coala supports only python 3.3 or later.")
        exit(4)



import os

from coalib.output.dbus.DbusDocument import DbusDocument


class DbusApp:
    """
    Stores data about each client that connects to the DbusServer
    """

    def __init__(self, app_id, name=""):
        self.app_id = app_id
        self.name = name

        self.docs = {}
        self.__next_doc_id = 0

    def _next_doc_id(self):
        self.__next_doc_id += 1
        return self.__next_doc_id

    def create_document(self, path):
        """
        Create a new dbus document.

        :param path:        The path to the document to be created.
        :param object_path: The dbus object path to use as the base for the
                            document object path.
        :param object_path: The connection to which the new ddocument object
                            path should be added.
        :return:            a DbusDocument object.
        """
        path = os.path.abspath(os.path.expanduser(path))
        doc = DbusDocument(doc_id=self._next_doc_id(), path=path)
        self.docs[path] = doc

        return doc

    def dispose_document(self, path):
        """
        Dispose of the document with the given path. It fails silently if the
        document does not exist. If there are no more documents in the app,
        the app is disposed.

        :param path: The path to the document.
        """
        path = os.path.abspath(os.path.expanduser(path))
        try:
            return self.docs.pop(path)
        except KeyError:
            return None





def typechain(*args):
    """
    Returns function which applies the first transformation it can from args
    and returns transformed value, or the value itself if it is in args.

    >>> function = typechain(int, 'a', ord, None)
    >>> function("10")
    10
    >>> function("b")
    98
    >>> function("a")
    'a'
    >>> function(int)
    <class 'int'>
    >>> function(None) is None
    True
    >>> function("str")
    Traceback (most recent call last):
        ...
    ValueError: Couldn't convert value 'str' to any specified type or find it \
in specified values.

    :raises TypeError:  Raises when either no functions are specified for
                        checking.
    """
    if len(args) == 0:
        raise TypeError("No arguments were provided.")

    def annotation(value):
        """
        Returns value either transformed with one of the function in args, or
        casted to one of types in args, or the value itself if it is in the
        args.

        :raises ValueError: Raises when cannot transform value in any one of
                            specified ways.
        """
        for arg in args:
            if value == arg:
                return value
            if isinstance(arg, type) and isinstance(value, arg):
                return value
            try:
                return arg(value)
            except (ValueError, TypeError):
                pass
        raise ValueError(
            "Couldn't convert value {value} to any specified type "
            "or find it in specified values.".format(
                value=repr(value)))
    return annotation
